/**************************************************************
CODES.H					Board No.: 79049

    Status/error codes and other such definitions
    Address map definitions of external I/O

    Date    13/12/99

    Author  Paul Carpenter

Version History
Rev     Date     Author          Changes
1.00    14/12/99 Paul Carpenter  Original Version
***************************************************************/
#define BYTE    unsigned char
#define WORD    unsigned short

#define NO_COMMANDS     13
#define MAX_PARAM       5

/* Number of processes in main loop timer array */
#define MAX_EVENTS      5
#define HEAD            0
#define LIGHT           1
#define FAN             2
#define ANALOG          3
#define COMMS           4

/** SYSTEM Status (sys_status) codes definitions **/
#define SW_LEFT         0x01
#define SW_RIGHT        0x02
#define SW_STEREO       0x03
#define CONT_CHANGE     0x04
#define BRIGHT_CHANGE   0x08
#define LIGHT_OK        0x10
#define HEAD_DET        0x40
#define SYS_ERROR       0x80

/** Status codes definitions **/
                                // Normal Running
#define STAT_OK         0x10
                                /* Tracking status */
#define STEREO          0x20
#define SWITCH_LEFT     0x21
#define SWITCH_RIGHT    0x22
#define NO_TRACK_LEFT   0x23
#define NO_TRACK_RIGHT  0x24

/* Setup Mode status */
#define SETUP_OK        0x40
#define SETUP_END       0x41

/* Flash mode status */
#define FLASH_OK        0x50

/**** ERROR CODES ****/
                                /* startup and checksum errors */
#define EXTRAM_COUNT    0x81
#define EXTRAM_ZERO     0x82
#define INT_PORT        0x83
#define CONFIGURE       0x84
#define INTRAM_COUNT    0x85
#define INTRAM_ZERO     0x86
#define BAD_NVRAM       0x87
                                /* Command parsing errors */
#define BAD_COMMAND     0xC0
/* The following are ADDED to BAD_COMMAND for type of error */
#define NO_PARAM1       0x1
#define NO_PARAM2       0x2
#define NO_PARAM3       0x3
#define BAD_PARAM1      0x4
#define BAD_PARAM2      0x5
#define BAD_PARAM3      0x6
#define PARAM_OVERFLOW  0x7
#define BAD_MODE        0xD
                            /* Hardware failure on verify a command action */
#define HARDWARE_FAIL   0xF

                                /* Binary mode comms errors */
#define BAD_START       0xE0
#define BAD_CHECKSUM    0xE1
#define BAD_ABORT       0xE2
#define INV_FORMAT      0xE3

/* System modes definitions */
#define NORMAL          0x20
#define SETUP           0x40
#define FLASH           0x80

/* system global types */
#define ON              1
#define OFF             0

#define IOUT_MIN        0
#define IOUT_MAX        255

/* Buffer sizes */
#define BINARY_SIZE     128
#define RX_SIZE         32
#define TX_SIZE         64
#define MAX_RATES       6

/* Status bit defines */
#define FRAME           0x80
#define HEAD_FOUND      0x40
#define BREAK_DET       0x20
#define TX_ERROR        0x10
#define RX_ERROR        0x08
#define TX_PENDING      0x04
#define RX_PENDING      0x02
#define BINARY_MODE     0x01

/* Mode bits define */
#define SIO_ENABLE      0x80
#define COMM_PORT       0x40
#define HEAD_PORT       0x20
#define RATE_SELECT     0x07

/* Address type definitions for SETUP/Flash mode addresses */
#define READ            0
#define WRITE           1
#define FLASH_T         2

/****************** Address Map of I/O ****************************/
/* Memory blocks and special memory start/stop address */
#define RAM_START       ( *(BYTE *) 0x0FFEF20 )
#define RAM_TOP         ( *(BYTE *) 0x0FFFF1F )

