/***************************************************************************
  File: Console.c

  created: 8/1/95, PBB

    Stolen from Hitachi examples

    Adapted and added to 9/6/98 PC

  description:
 
  Simple console I/O routines for serial port 1.
  Makes use of the hardware register description header file
  h83048f.h

	***** This code assumes that the hardware has been configured
	***** and initialised at start_up
***************************************************************************/
#include <stdarg.h>
#include <ctype.h>
#include <stdlib.h>   
#include <string.h>

#include "h83067f.h"
#include "ascii.h"
#include "codes.h"
#include "h8core_b.h"
#include "parse.h"
#include "serial.h"

char tochar[] __attribute__ ((section (".strings"))) =
    "0123456789ABCDEF";
char crlf[] __attribute__ ((section (".strings"))) =
    "\n\r";

/***** Prototypes defined here **************/
int  PrHexNumber( unsigned long, int, int );
int  PrDecNumber( long );
int  GenHexNumber( char *, unsigned long, int, char );
int  mon_printf( char *,... );


/********************************************/
/*  GenHexNumber Create a HEX string        */
/*                                          */
/*  Inputs  - pointer to buffer to hold     */
/*              resultant string            */
/*          - long value to convert         */
/*          - int printed field width       */
/*          - char ASCII char to pad field  */
/*                                          */
/*  Output  int Number of characters in     */
/*              unterminated output string  */
/*                                          */
/*  Function normally called from mon_printf*/
/*  via PrHexNumber                         */
/********************************************/
int GenHexNumber( char *ptr, unsigned long a, int width, char pad )
{
int idx = 0;
int i;
char buf[ 20 ];

do
  {
  buf[ idx++ ] = tochar[ a & 0xF ];
  a >>= 4;
  }
while( a );
if( width >= 0 )                        /* Pad upto width ? */
  for( i = idx; i < width; i++ )
     putchar( 1, pad );
for( i = idx; i; i-- )                  /* Fill buffer in reverse order */
   *ptr++ = buf[ i - 1 ];
return idx;
}


/********************************************/
/*  PrHexNumber Print a HEX string          */
/*                                          */
/*  Inputs  - long value to convert         */
/*          - int printed field width       */
/*          - char ASCII char to pad field  */
/*                                          */
/*  Output  int Number of characters in     */
/*              unterminated output string  */
/*                                          */
/*  Function normally caled from mon_printf */
/*  via PrHexNumber                         */
/********************************************/
int PrHexNumber( unsigned long a, int width, int pad )
{
int i;
int j;
char buf[ 10 ];

i = GenHexNumber( buf, a, width, pad );
for( j = 0; j < i; j++ )
   putchar( 1, buf[ j ] );
return i;
}


/********************************************/
/*  PrDecNumber Print decimal number string */
/*                                          */
/*  Inputs  - long value to convert         */
/*                                          */
/*  Output  int Number of characters in     */
/*              unterminated output string  */
/*                                          */
/*  Function normally caled from mon_printf */
/********************************************/
int PrDecNumber( long a )
{
char buf[ 10 ];
int idx = 0;
int i;
int m = 0;
long d = 10;
int k;

if( a < 0 )
  {
  d = -10;
  m = 1;
  }
do
  {
  k = (int)(a % d);
  if( k < 0 )
	k = -k;
  buf[ idx++ ] = tochar[ k ];
  a /= d;
  }
while( a );
if( m )
  buf[ idx++ ] = '-';
for( i = idx; i; i-- )
   putchar( 1, buf[ i - 1 ] );
return idx;
}


/*--------------------------------------------------------------*/
/* Printf routine                                               */
/*                                                              */
/* int printf(char *format, ... )                               */
/*                                                              */
/* return: number of characters printed                         */
/*--------------------------------------------------------------*/
int mon_printf( char *format,... )
{
va_list arg_ptr;              /* arguments */
int ret_val = 0;
va_start( arg_ptr, format );
while( 1 )
  {
  switch( *format++ )
    {
	default:
      putchar( 1, format[ -1 ] );
	  ret_val++;
	  break;
	case 0:
	  return ret_val;
	case '%':
	  {
	  int along = 0;
	  int leading_zeros = ' ';
	  int width = -1;
	  /* See what modifiers we have - */
	  if( *format == '0' )
	    {
	    leading_zeros = '0';
	    format++;
	    }
	  if( *format >= '0' && *format <= '9' )
	    {
	    width = *format - '0';
	    format++;
	    }
	  if( *format == 'l' )
	    {
	    along = 1;
	    format++;
	    }
      switch( *format++ )
	     {
	     case 'x':
	     case 'X':
                ret_val += PrHexNumber( along ? va_arg( arg_ptr, unsigned long )
                                            : va_arg( arg_ptr, unsigned int ),
                                                        width, leading_zeros );
                break;
	     case 'p':
                ret_val += PrHexNumber( (unsigned long ) va_arg( arg_ptr,
                                                        unsigned char * ),
                                        sizeof( unsigned char * ) * 2, '0' );
                break;
	     case 'c':
                putchar( 1, va_arg( arg_ptr, unsigned int ) );
                ret_val++;
                break;
	     case 's':
                {
                char *src = va_arg( arg_ptr, char * );
                ret_val += putstring( 1, src );
                break;
                }
	     case 'd':
                ret_val += PrDecNumber( along ? va_arg( arg_ptr, long )
                                                : va_arg( arg_ptr, int ) );
                break;
              }
	  }
    }
  }
}

