/* writebyte.S : write one byte to EEPROM */
; delay values for a 16 MHz quartz
;a=40 b=12 c=12 d=1516 e=12 f=15 g=15

; SCI RX1 interrupt vector is at 00E4-00E7 (4 bytes)
; MSB of @ must be stored at 00E4, LSB at 00E7 (cf memory format p. 25)

#include "h83048addr.h"

#define AAAA #0x28
#define GGGG #0x0F
#define BBBB #0x0C
#define WDTVAL #0xA57F
			; A5=passwd for accessing TCSR (p. 447 of manual)
; writebyte : write 1 byte to EEPROM
; int writebyte(int addr,unsigned char* value)
;
	.h8300h
;	.section .text
	.align 1
	.global _start
_start:	
        mov.l   #0xffefc,er7	; =mov.l ...,sp : set stack pointer
        ldc.b   #0xc0,  ccr     ; disable interrupt
     ;   mov.b   #0x00,  r0l     ; 2-State Access(All area)
     ;   mov.b   r0l,@0xffffed

        mov.b   #0xff,  r0l     ; port 5 as output
        mov.b   r0l,@0xfffc8

	bset	#4,@0xfffba:8	; SCI initialization
	bset	#5,@0xfffba:8	; |	
	bset	#6,@0xfffba:8	; |	<- enable Rx1 interrupt
	mov.w	#0xff00,r0	; |
	mov.b	r0h,@0xfffd4:24	; |

	bset	#0,@0xfffca:8	; P5DR et C8 -> FF pour out:
	bclr	#1,@0xfffca:8	; P5DR et C8 -> FF pour out:
	jsr	DELAI
	bclr	#0,@0xfffca:8	; P5DR et C8 -> FF pour out:
	bset	#1,@0xfffca:8	; P5DR et C8 -> FF pour out:
	jsr	DELAI
	mov.l	#0x000000e7,er2:32	; END @ (ie we write @-3 to @) 
	mov.l	#rx1isr,er3		; data
	mov 	#04,r0l

continue:
	push	er0	; no need to save er2/er3: not modified by wrtbyte
	jsr	wrtbyte	
	bsr	snd
	mov	r3l,r0l
	bsr	snd
	pop	er0
	dec.l	#1,er2	; WARNING: this is a DEC, not an INC
	shlr.l	er3
	shlr.l	er3
	shlr.l	er3
	shlr.l	er3
	shlr.l	er3
	shlr.l	er3
	shlr.l	er3
	shlr.l	er3
	dec	r0l
	bne continue

	mov.l	@0x000000e4,er2	; read content of 0x000000e4

	mov	r2l,r0l
	bsr	snd
	mov	r2h,r0l
	bsr	snd
	mov 	e2,r2
	mov	r2l,r0l
	bsr	snd
	mov	r2h,r0l
	bsr	snd

	andc.b	#0x7F,ccr	; I bit set to 0 => enable IRQs

	mov.b	#0,r0l		; why am I not allowed to bclr.b #1,@0x0ff300 ?
	mov.b	r0l,@0xff300	; prepare register indicating an IRQ occured
mainloop:bset   #1,@0xfffca:8
	jsr     DELAI
	bclr    #1,@0xfffca:8
	jsr     DELAI
	mov.b	@0xff300,r0l
	cmp.b	#0x00,r0l
	beq	mainloop	; if bit of r1l==0, no IRQ => no LED
	bset   	#0,@0xfffca:8	; otherwise blink second LED
	jsr     DELAI
	bclr    #0,@0xfffca:8
	jsr     DELAI
	mov.b	#0,r0l		; we received the char, now forget about it
	mov.b	r0l,@0xff300	; |
	jmp	mainloop
	sleep		; THE END

snd: 	btst    #7,@0xfffbc:8
        beq     snd
        mov.b   r0l,@0xfffbb:8
	bclr    #7,@0xfffbc:8
	rts

DELAI:	mov.w   #0x00ff, r4 ; Set program loop counter mais 1f ne marche pas
WAI1:	mov.w   #0x0fff, r5 ; Set program loop counter     8fff
WAI:	dec.w   #1, r5 ; Program
	bpl	WAI
	dec.w   #1, r4
	bpl     WAI1
	rts

rx1isr:	push	r0
rcv:	btst    #6,@0xfffbc:8	; char received ?
        beq     rcv
	mov.b   @0xfffbd:8,r0l	; yes: read it
	bclr    #6,@0xfffbc:8
	mov.b	r0l,@0xff300	; store the received byte in 1st RAM @
	pop	r0
	rte

wrtbyte:			; ARGUMENTS: addr in er2, data in r3h
	mov.w	#0001,r0	;vprg-vrfy count (manual, p.592)
	mov.w	GGGG,r1 	;variable 'g'
	mov.w	#0x4140,r4	
	mov.b	r4l,@FLMCR:8

jmLOOP0:dec.w	#1, r1 ;
	bpl	jmLOOP0
	mov.b	#01,r0h		; EBR1 MUST BE 1 => we can write in 1st block
	mov.b	r0h, @EBR1:8 	; Set EBR1 (r2h=0)      (1st blk=00000-03ffff)
	mov.b	r3l, @er2 	; Dummy write (er1=@, r2l=data)

	mov.w	AAAA, e4 	; Set initial program loop counter value
PRGMS:	mov.w	WDTVAL, r4 	; Start watchdog timer
	mov.w	r4, @TCSR:16 	;
	mov.w	e4,e5
	mov.w	#0x4140, r4 	;
	mov.b	r4h, @FLMCR:8 	; Set P bit

jmLOOP1:dec.w	#1, e5 		; Program
	bpl	jmLOOP1
	mov.w	#0xA500, r4 	;
	mov.w	r4, @TCSR:16 	; Stop watchdog timer

	mov.w	BBBB , r1 	; Set program-verify loop counter
	mov.b	#0x44, r4h 	;
	mov.b	r4h, @FLMCR:8 	; Clear P bit
jmLOOP2:dec.w	#1, r1 		; Wait
	bpl	jmLOOP2

	mov.b	@er2,r1h
	cmp.b	r3l,r1h

	beq	PVOK 		; Program-verify decision
PVNG:	mov.b	#0x40, r5h 	;
	mov.b	r5h, @FLMCR:8 	; Clear PV bit
	cmp.b	#06, r0l 	; Program-verify executed 6 times?
	beq	NGEND 		; If prg-vrfy executed 6 times, branch to NGEND
	inc.b	r0l 		; Program-verify fail count + 1
	shll.w	e4 		; Double program loop counter value
	bra	PRGMS 		; Program again
PVOK:	mov.w	#0x4000, r5 	;
	mov.b	r5h, @FLMCR:8 	; Clear PV bit
	mov.b	r5l, @EBR1:8 	; Clear EBR1
	mov.b	r5l, @FLMCR:8 	; Clear V PP E bit One byte programmed
	sub	r0,r0
	bra	EXIT

NGEND:	mov.w	#0x4000, r5 ;
	mov.b	r5l, @EBR1:8 	; Clear EBR1
	MOV.B	r5l, @FLMCR:8 	; Clear VPP E bit
	mov.w	#0xfe,r0

EXIT:	; r0=-1 if programming failed, 0 if programming succeeded
	rts

	.end

; output of h8rec:
; 170($aa)   <- left over from the communication protocol
; 0($0)      <- ack writing 1 byte (would be $fe otherwise)
; 156($9c)   <- value of the byte just written
; 0($0)      <- repeat another 3 times
; 243($f3)   <- |
; 0($0)      <- |
; 15($f)     <- |
; 0($0)      <- |
; 0($0)      <- last byte written
; 156($9c)   <- start reading address we have just written, LSB 1st
; 243($f3)   <- |
; 15($f)     <- |
; 0($0)      <- end of reading, MSB last => we read 00 0f f3 9c = ISR @

