;
;       crt0.s
;
;       h8/300h  C runtime environment start up file.
;
;
        .h8300h
        .section .text
        .global _start

        .extern _hw_initialize
        .extern _etext                  ; ROM copy of initialized data segment
        .extern _data                   ; beginning of initialized data segment
        .extern _edata                  ; end of initialized data segment
        .extern _bss                    ; beginning of uninitialized data segment
        .extern _ebss                   ; end of uninitialised data segment
        .extern _stack                  ; stack segment


;
;       _start
;       system start
;
;       input:
;       <none>
;
;       output:
;       <none>
;
;       process:
;       init temp stack pointer
;       init hardware & chip selects
;       init real stack pointer
;       init initialised data
;       zero uninitialised data
;       mark stack & heap to show usage
;       call main
;
_start:
; C runtime start
        
;=========================================================================
; Set stack pointer initially to internal ram, external ram CS not active yet
;-------------------------------------------------------------------------
        mov.l   #0xFFFEFC,sp

;=========================================================================
; Setup H8/3003H Hardware & Chip selects
;-------------------------------------------------------------------------
        jsr     @_hw_initialize

;=========================================================================
; Now safe to set stack pointer to external ram
; Interrupts should not be enabled yet
;-------------------------------------------------------------------------
        mov.l   #_stack,sp

;=========================================================================
; copy initialized .data from ROM to RAM data area
;=========================================================================
        mov.l   #_data,er0              ; start of RAM .data section
        mov.l   #_edata,er1             ; end of RAM .data section
        mov.l   #_etext,er2             ; location of ROM .data storage
data_init_loop:
        mov.w   @er2,r3                 ; get next word
        mov.w   r3,@er0                 ; move it
        adds    #2,er2                  ; increment source ptr
        adds    #2,er0                  ; increment destination ptr
        cmp.l   er1,er0                 ; test for done
        blo     data_init_loop
        
;=========================================================================
; zero out .bss ram data area
;-------------------------------------------------------------------------
        mov.l   #_bss,er0               ; start of RAM .bss section
        mov.l   #_ebss,er1              ; end of RAM .bss section
        mov.w   #0,r2
bss_init_loop:
        mov.w   r2,@er0                 ; clear memory
        adds    #2,er0                  ; increment ptr
        cmp.l   er1,er0                 ; test for done
        blo     bss_init_loop

;=========================================================================
; initialize external stack area to "DEAD" to track stack usage
;-------------------------------------------------------------------------
        mov.l   #_end,er0               ; end of RAM data areas
        mov.l   #_stack,er1             ; end of RAM .stack section
        mov.w   #0xDEAD,r2              ; marker indicating unused stack
stack_init_loop:
        mov.w   r2,@er0                 ; store marker
        adds    #2,er0                  ; increment ptr
        cmp.l   er1,er0                 ; test for done
        blo     stack_init_loop

; call main
        jsr     @_main

;=========================================================================
; on return from main, call exit
;-------------------------------------------------------------------------
_exit:
        sleep
        bra     _exit
                        
        .end
