/*  
 * h3003.h
 *
 * H8/3003 register space definitions
 *
 * The register space starts at: 
 *   0x0fff1c   in Modes 1 or 2 (1-MByte address space)
 *   0xffff1c   in Modes 3 or 4 (16-MByte address space)
 * It seems that using the 16-MByte form works in all modes, so
 * that form is given here.  
 *
 * Other points:
 * -- Only byte references are given.
 * -- Unassigned registers are given place forms of the form "reg_nn"
 *    where nn = the rightmost digits of the address 
 * -- The manual gives both SCI channel registers ambiguously, so the
 *    a numbered notation has been added here.
 * -- There is some addressing strangeness for NDRA and NDRB in the
 *    region 0xA4...0xA7. See the manual for particulars.
 * 
 * For details see, for example, H8/3003 Hardware Manual Appendix B
 * (where only the 2 LSDs of the addresses are given).
 *
 * Warning
 * -------
 * Entering this data is incredibly boring.  You are invited to check
 * the values given herein for accuracy, because there may well be errors.
 * 
 */

/* DMAC channel 0A */
#define reg_1c      (*(volatile unsigned char  *)(0xffff1c))
#define reg_1d      (*(volatile unsigned char  *)(0xffff1d))
#define reg_1e      (*(volatile unsigned char  *)(0xffff1e))
#define reg_1f      (*(volatile unsigned char  *)(0xffff1f))

#define MAR0A       (*(volatile unsigned long  *)(0xffff20))
#define MAR0AR      (*(volatile unsigned char  *)(0xffff20))
#define MAR0AE      (*(volatile unsigned char  *)(0xffff21))
#define MAR0AH      (*(volatile unsigned char  *)(0xffff22))
#define MAR0AL      (*(volatile unsigned char  *)(0xffff23))
#define ETCR0A      (*(volatile unsigned short *)(0xffff24))
#define ETCR0AH     (*(volatile unsigned char  *)(0xffff24))
#define ETCR0AL     (*(volatile unsigned char  *)(0xffff25))
#define IOAR0A      (*(volatile unsigned char  *)(0xffff26))
#define DTCR0A      (*(volatile unsigned char  *)(0xffff27))

/* DMAC channel 0B */
#define MAR0B       (*(volatile unsigned long  *)(0xffff28))
#define MAR0BR      (*(volatile unsigned char  *)(0xffff28))
#define MAR0BE      (*(volatile unsigned char  *)(0xffff29))
#define MAR0BH      (*(volatile unsigned char  *)(0xffff2a))
#define MAR0BL      (*(volatile unsigned char  *)(0xffff2b))
#define ETCR0B      (*(volatile unsigned short *)(0xffff2c))
#define ETCR0BH     (*(volatile unsigned char  *)(0xffff2c))
#define ETCR0BL     (*(volatile unsigned char  *)(0xffff2d))
#define IOAR0B      (*(volatile unsigned char  *)(0xffff2e))
#define DTCR0B      (*(volatile unsigned char  *)(0xffff2f))

/* DMAC channel 1A */
#define MAR1A       (*(volatile unsigned long  *)(0xffff30))
#define MAR1AR      (*(volatile unsigned char  *)(0xffff30))
#define MAR1AE      (*(volatile unsigned char  *)(0xffff31))
#define MAR1AH      (*(volatile unsigned char  *)(0xffff32))
#define MAR1AL      (*(volatile unsigned char  *)(0xffff33))
#define ETCR1A      (*(volatile unsigned short *)(0xffff34))
#define ETCR1AH     (*(volatile unsigned char  *)(0xffff34))
#define ETCR1AL     (*(volatile unsigned char  *)(0xffff35))
#define IOAR1A      (*(volatile unsigned char  *)(0xffff36))
#define DTCR1A      (*(volatile unsigned char  *)(0xffff37))

/* DMAC channel 1B */
#define MAR1B       (*(volatile unsigned long  *)(0xffff38))
#define MAR1BR      (*(volatile unsigned char  *)(0xffff38))
#define MAR1BE      (*(volatile unsigned char  *)(0xffff39))
#define MAR1BH      (*(volatile unsigned char  *)(0xffff3a))
#define MAR1BL      (*(volatile unsigned char  *)(0xffff3b))
#define ETCR1B      (*(volatile unsigned short *)(0xffff3c))
#define ETCR1BH     (*(volatile unsigned char  *)(0xffff3c))
#define ETCR1BL     (*(volatile unsigned char  *)(0xffff3d))
#define IOAR1B      (*(volatile unsigned char  *)(0xffff3e))
#define DTCR1B      (*(volatile unsigned char  *)(0xffff3f))

/* DMAC channel 2A */
#define MAR2A       (*(volatile unsigned long  *)(0xffff40))
#define MAR2AR      (*(volatile unsigned char  *)(0xffff40))
#define MAR2AE      (*(volatile unsigned char  *)(0xffff41))
#define MAR2AH      (*(volatile unsigned char  *)(0xffff42))
#define MAR2AL      (*(volatile unsigned char  *)(0xffff43))
#define ETCR2A      (*(volatile unsigned short *)(0xffff44))
#define ETCR2AH     (*(volatile unsigned char  *)(0xffff44))
#define ETCR2AL     (*(volatile unsigned char  *)(0xffff45))
#define IOAR2A      (*(volatile unsigned char  *)(0xffff46))
#define DTCR2A      (*(volatile unsigned char  *)(0xffff47))

/* DMAC channel 2B */
#define MAR2B       (*(volatile unsigned long  *)(0xffff48))
#define MAR2BR      (*(volatile unsigned char  *)(0xffff48))
#define MAR2BE      (*(volatile unsigned char  *)(0xffff49))
#define MAR2BH      (*(volatile unsigned char  *)(0xffff4a))
#define MAR2BL      (*(volatile unsigned char  *)(0xffff4b))
#define ETCR2B      (*(volatile unsigned short *)(0xffff4c))
#define ETCR2BH     (*(volatile unsigned char  *)(0xffff4c))
#define ETCR2BL     (*(volatile unsigned char  *)(0xffff4d))
#define IOAR2B      (*(volatile unsigned char  *)(0xffff4e))
#define DTCR2B      (*(volatile unsigned char  *)(0xffff4f))

/* DMAC channel 3A */
#define MAR3A       (*(volatile unsigned long  *)(0xffff50))
#define MAR3AR      (*(volatile unsigned char  *)(0xffff50))
#define MAR3AE      (*(volatile unsigned char  *)(0xffff51))
#define MAR3AH      (*(volatile unsigned char  *)(0xffff52))
#define MAR3AL      (*(volatile unsigned char  *)(0xffff53))
#define ETCR3A      (*(volatile unsigned short *)(0xffff54))
#define ETCR3AH     (*(volatile unsigned char  *)(0xffff54))
#define ETCR3AL     (*(volatile unsigned char  *)(0xffff55))
#define IOAR3A      (*(volatile unsigned char  *)(0xffff56))
#define DTCR3A      (*(volatile unsigned char  *)(0xffff57))

/* DMAC channel 3B */
#define MAR3B       (*(volatile unsigned long  *)(0xffff58))
#define MAR3BR      (*(volatile unsigned char  *)(0xffff58))
#define MAR3BE      (*(volatile unsigned char  *)(0xffff59))
#define MAR3BH      (*(volatile unsigned char  *)(0xffff5a))
#define MAR3BL      (*(volatile unsigned char  *)(0xffff5b))
#define ETCR3B      (*(volatile unsigned short *)(0xffff5c))
#define ETCR3BH     (*(volatile unsigned char  *)(0xffff5c))
#define ETCR3BL     (*(volatile unsigned char  *)(0xffff5d))
#define IOAR3B      (*(volatile unsigned char  *)(0xffff5e))
#define DTCR3B      (*(volatile unsigned char  *)(0xffff5f))

/* 16-bit Integrated Timer-pulse Unit (ITU) (all channels) */
#define TSTR        (*(volatile unsigned char  *)(0xffff60))
#define TSNCA       (*(volatile unsigned char  *)(0xffff61))
#define TMDR        (*(volatile unsigned char  *)(0xffff62))
#define TFCRAL      (*(volatile unsigned char  *)(0xffff63))

/* 16-bit Integrated Timer-pulse Unit (ITU) (channel 0) */
#define TCR0        (*(volatile unsigned char  *)(0xffff64))
#define TIOR0AL     (*(volatile unsigned char  *)(0xffff65))
#define TIER0       (*(volatile unsigned char  *)(0xffff66))
#define TSR0        (*(volatile unsigned char  *)(0xffff67))
#define TCNT0       (*(volatile unsigned short *)(0xffff68))
#define TCNT0H      (*(volatile unsigned char  *)(0xffff68))
#define TCNT0L      (*(volatile unsigned char  *)(0xffff69))
#define GRA0        (*(volatile unsigned short *)(0xffff6a))
#define GRA0H       (*(volatile unsigned char  *)(0xffff6a))
#define GRA0L       (*(volatile unsigned char  *)(0xffff6b))
#define GRB0        (*(volatile unsigned short *)(0xffff6c))
#define GRB0H       (*(volatile unsigned char  *)(0xffff6c))
#define GRB0L       (*(volatile unsigned char  *)(0xffff6d))

/* 16-bit Integrated Timer-pulse Unit (ITU) (channel 1) */
#define TCR1        (*(volatile unsigned char  *)(0xffff6e))
#define TIOR1       (*(volatile unsigned char  *)(0xffff6f))
#define TIER1       (*(volatile unsigned char  *)(0xffff70))
#define TSR1        (*(volatile unsigned char  *)(0xffff71))
#define TCNT1       (*(volatile unsigned short *)(0xffff72))
#define TCNT1H      (*(volatile unsigned char  *)(0xffff72))
#define TCNT1L      (*(volatile unsigned char  *)(0xffff73))
#define GRA1        (*(volatile unsigned short *)(0xffff74))
#define GRA1H       (*(volatile unsigned char  *)(0xffff74))
#define GRA1L       (*(volatile unsigned char  *)(0xffff75))
#define GRB1        (*(volatile unsigned short *)(0xffff76))
#define GRB1H       (*(volatile unsigned char  *)(0xffff76))
#define GRB1L       (*(volatile unsigned char  *)(0xffff77))

/* 16-bit Integrated Timer-pulse Unit (ITU) (channel 2) */
#define TCR2        (*(volatile unsigned char  *)(0xffff78))
#define TIOR2       (*(volatile unsigned char  *)(0xffff79))
#define TIER2       (*(volatile unsigned char  *)(0xffff7a))
#define TSR2        (*(volatile unsigned char  *)(0xffff7b))
#define TCNT2       (*(volatile unsigned short *)(0xffff7c))
#define TCNT2H      (*(volatile unsigned char  *)(0xffff7c))
#define TCNT2L      (*(volatile unsigned char  *)(0xffff7d))
#define GRA2        (*(volatile unsigned short *)(0xffff7e))
#define GRA2H       (*(volatile unsigned char  *)(0xffff7e))
#define GRA2L       (*(volatile unsigned char  *)(0xffff7f))
#define GRB2        (*(volatile unsigned short *)(0xffff80))
#define GRB2H       (*(volatile unsigned char  *)(0xffff80))
#define GRB2L       (*(volatile unsigned char  *)(0xffff81))

/* 16-bit Integrated Timer-pulse Unit (ITU) (channel 3) */
#define TCR3        (*(volatile unsigned char  *)(0xffff82))
#define TIOR3       (*(volatile unsigned char  *)(0xffff83))
#define TIER3       (*(volatile unsigned char  *)(0xffff84))
#define TSR3        (*(volatile unsigned char  *)(0xffff85))
#define TCNT3       (*(volatile unsigned short *)(0xffff86))
#define TCNT3H      (*(volatile unsigned char  *)(0xffff86))
#define TCNT3L      (*(volatile unsigned char  *)(0xffff87))
#define GRA3        (*(volatile unsigned short *)(0xffff88))
#define GRA3H       (*(volatile unsigned char  *)(0xffff88))
#define GRA3L       (*(volatile unsigned char  *)(0xffff89))
#define GRB3        (*(volatile unsigned short *)(0xffff8a))
#define GRB3H       (*(volatile unsigned char  *)(0xffff8a))
#define GRB3L       (*(volatile unsigned char  *)(0xffff8b))
#define BRA3        (*(volatile unsigned short *)(0xffff8c))
#define BRA3H       (*(volatile unsigned char  *)(0xffff8c))
#define BRA3L       (*(volatile unsigned char  *)(0xffff8d))
#define BRB3        (*(volatile unsigned short *)(0xffff8e))
#define BRB3H       (*(volatile unsigned char  *)(0xffff8e))
#define BRB3L       (*(volatile unsigned char  *)(0xffff8f))

/* 16-bit Integrated Timer-pulse Unit (ITU) (all channels) */
#define TOER        (*(volatile unsigned char  *)(0xffff90))
#define TOCR        (*(volatile unsigned char  *)(0xffff91))

/* 16-bit Integrated Timer-pulse Unit (ITU) (channel 4) */
#define TCR4        (*(volatile unsigned char  *)(0xffff92))
#define TIOR4       (*(volatile unsigned char  *)(0xffff93))
#define TIER4       (*(volatile unsigned char  *)(0xffff94))
#define TSR4        (*(volatile unsigned char  *)(0xffff95))
#define TCNT4       (*(volatile unsigned short *)(0xffff96))
#define TCNT4H      (*(volatile unsigned char  *)(0xffff96))
#define TCNT4L      (*(volatile unsigned char  *)(0xffff97))
#define GRA4        (*(volatile unsigned short *)(0xffff98))
#define GRA4H       (*(volatile unsigned char  *)(0xffff98))
#define GRA4L       (*(volatile unsigned char  *)(0xffff99))
#define GRB4        (*(volatile unsigned short *)(0xffff9a))
#define GRB4H       (*(volatile unsigned char  *)(0xffff9a))
#define GRB4L       (*(volatile unsigned char  *)(0xffff9b))
#define BRA4        (*(volatile unsigned short *)(0xffff9c))
#define BRA4H       (*(volatile unsigned char  *)(0xffff9c))
#define BRA4L       (*(volatile unsigned char  *)(0xffff9d))
#define BRB4        (*(volatile unsigned short *)(0xffff9e))
#define BRB4H       (*(volatile unsigned char  *)(0xffff9e))
#define BRB4L       (*(volatile unsigned char  *)(0xffff9f))

/* Timing Pattern Controller (TPC) */
#define TPMR        (*(volatile unsigned char  *)(0xffffa0))
#define TPCR        (*(volatile unsigned char  *)(0xffffa1))
#define NDERB       (*(volatile unsigned char  *)(0xffffa2))
#define NDERA       (*(volatile unsigned char  *)(0xffffa3))
#define NDRB        (*(volatile unsigned char  *)(0xffffa4))
#define NDRA_AGAIN  (*(volatile unsigned char  *)(0xffffa5))
#define NDRB_AGAIN  (*(volatile unsigned char  *)(0xffffa6))
#define NDRA        (*(volatile unsigned char  *)(0xffffa7))

/* WatchDog Timer (WDT) */
#define TCSR        (*(volatile unsigned char  *)(0xffffa8))
#define TCNT        (*(volatile unsigned char  *)(0xffffa9))
#define reg_aa      (*(volatile unsigned char  *)(0xffffaa))

/* Refresh Controller */
#define RSTCSR      (*(volatile unsigned char  *)(0xffffab))
#define RFSHCR      (*(volatile unsigned char  *)(0xffffac))
#define RTMCSR      (*(volatile unsigned char  *)(0xffffad))
#define RTCNT       (*(volatile unsigned char  *)(0xffffae))
#define RTCOR       (*(volatile unsigned char  *)(0xffffaf))

/* SCI Channel 0 */
#define SMR0        (*(volatile unsigned char  *)(0xffffb0)) /* serial mode reg */  
#define BRR0        (*(volatile unsigned char  *)(0xffffb1)) /* bit rate reg */    
#define SCR0        (*(volatile unsigned char  *)(0xffffb2)) /* serial control reg */
#define TDR0        (*(volatile unsigned char  *)(0xffffb3)) /* transmit data reg */
#define SSR0        (*(volatile unsigned char  *)(0xffffb4)) /* serial status reg */
#define RDR0        (*(volatile unsigned char  *)(0xffffb5)) /* receive data reg */
#define reg_b6      (*(volatile unsigned char  *)(0xffffb6))
#define reg_b7      (*(volatile unsigned char  *)(0xffffb7))

/* SCI Channel 1 */
#define SMR1        (*(volatile unsigned char  *)(0xffffb8)) /* serial mode reg */   
#define BRR1        (*(volatile unsigned char  *)(0xffffb9)) /* bit rate reg */      
#define SCR1        (*(volatile unsigned char  *)(0xffffba)) /* serial control reg */
#define TDR1        (*(volatile unsigned char  *)(0xffffbb)) /* transmit data reg */ 
#define SSR1        (*(volatile unsigned char  *)(0xffffbc)) /* serial status reg */ 
#define RDR1        (*(volatile unsigned char  *)(0xffffbd)) /* receive data reg */  
#define reg_be      (*(volatile unsigned char  *)(0xffffbe))
#define reg_bf      (*(volatile unsigned char  *)(0xffffbf))
             
#define reg_c0      (*(volatile unsigned char  *)(0xffffc0))
#define reg_c1      (*(volatile unsigned char  *)(0xffffc1))
#define reg_c2      (*(volatile unsigned char  *)(0xffffc2))
#define reg_c3      (*(volatile unsigned char  *)(0xffffc3))
#define reg_c4      (*(volatile unsigned char  *)(0xffffc4))

/* I/O ports 4,5,6,7,8,9,A,B,C */
#define P4DDR       (*(volatile unsigned char  *)(0xffffc5))
#define reg_c6      (*(volatile unsigned char  *)(0xffffc6))
#define P4DR        (*(volatile unsigned char  *)(0xffffc7))
#define P5DDR       (*(volatile unsigned char  *)(0xffffc8))
#define P6DDR       (*(volatile unsigned char  *)(0xffffc9))
#define P5DR        (*(volatile unsigned char  *)(0xffffca))
#define P6DR        (*(volatile unsigned char  *)(0xffffcb))
#define reg_cc      (*(volatile unsigned char  *)(0xffffcc))
#define P8DDR       (*(volatile unsigned char  *)(0xffffcd))
#define P7DR        (*(volatile unsigned char  *)(0xffffce))
#define P8DR        (*(volatile unsigned char  *)(0xffffcf))
#define P9DDR       (*(volatile unsigned char  *)(0xffffd0))
#define PADDR       (*(volatile unsigned char  *)(0xffffd1))
#define P9DR        (*(volatile unsigned char  *)(0xffffd2))
#define PADR        (*(volatile unsigned char  *)(0xffffd3))
#define PBDDR       (*(volatile unsigned char  *)(0xffffd4))
#define PCDDR       (*(volatile unsigned char  *)(0xffffd5))
#define PBDR        (*(volatile unsigned char  *)(0xffffd6))
#define PCDR        (*(volatile unsigned char  *)(0xffffd7))
#define reg_d8      (*(volatile unsigned char  *)(0xffffd8))
#define reg_d9      (*(volatile unsigned char  *)(0xffffd9))
#define P4PCR       (*(volatile unsigned char  *)(0xffffda))
#define P5PCR       (*(volatile unsigned char  *)(0xffffdb))
#define reg_dc      (*(volatile unsigned char  *)(0xffffdc))
#define reg_dd      (*(volatile unsigned char  *)(0xffffdd))
#define reg_de      (*(volatile unsigned char  *)(0xffffde))
#define reg_df      (*(volatile unsigned char  *)(0xffffdf))

/* A-to-D */
/* NOTE: reading a 16 bit A-to-D register as an unsigned
 * short may not function correctly.  The register may have
 * to be read using two 8 bit accesses.  See section 14.3
 */
#define ADDRA       (*(volatile unsigned short *)(0xffffe0))
#define ADDRAH      (*(volatile unsigned char  *)(0xffffe0))
#define ADDRAL      (*(volatile unsigned char  *)(0xffffe1))
#define ADDRB       (*(volatile unsigned short *)(0xffffe2))
#define ADDRBH      (*(volatile unsigned char  *)(0xffffe2))
#define ADDRBL      (*(volatile unsigned char  *)(0xffffe3))
#define ADDRC       (*(volatile unsigned short *)(0xffffe4))
#define ADDRCH      (*(volatile unsigned char  *)(0xffffe4))
#define ADDRCL      (*(volatile unsigned char  *)(0xffffe5))
#define ADDRD       (*(volatile unsigned short *)(0xffffe6))
#define ADDRDH      (*(volatile unsigned char  *)(0xffffe6))
#define ADDRDL      (*(volatile unsigned char  *)(0xffffe7))

#define ADCSR       (*(volatile unsigned char  *)(0xffffe8))
#define ADCR        (*(volatile unsigned char  *)(0xffffe9))
#define reg_ea      (*(volatile unsigned char  *)(0xffffea))
#define reg_eb      (*(volatile unsigned char  *)(0xffffeb))

/* Bus and system controllers */
#define ABWCR       (*(volatile unsigned char  *)(0xffffec))
#define ASTCR       (*(volatile unsigned char  *)(0xffffed))
#define WCR         (*(volatile unsigned char  *)(0xffffee))
#define WCER        (*(volatile unsigned char  *)(0xffffef))
#define reg_f0      (*(volatile unsigned char  *)(0xfffff0))
#define MDCR        (*(volatile unsigned char  *)(0xfffff1))
#define SYSCR       (*(volatile unsigned char  *)(0xfffff2))
#define BRCR        (*(volatile unsigned char  *)(0xfffff3))

/* Interrupt controller */
#define ISCR        (*(volatile unsigned char  *)(0xfffff4))
#define IER         (*(volatile unsigned char  *)(0xfffff5))
#define ISR         (*(volatile unsigned char  *)(0xfffff6))
#define reg_f7      (*(volatile unsigned char  *)(0xfffff7))

#define IPRA        (*(volatile unsigned char  *)(0xfffff8))
#define IPRB        (*(volatile unsigned char  *)(0xfffff9))
#define reg_fa      (*(volatile unsigned char  *)(0xfffffa))
#define reg_fb      (*(volatile unsigned char  *)(0xfffffb))
#define reg_fc      (*(volatile unsigned char  *)(0xfffffc))
#define reg_fd      (*(volatile unsigned char  *)(0xfffffd))
#define reg_fe      (*(volatile unsigned char  *)(0xfffffe))
#define reg_ff      (*(volatile unsigned char  *)(0xffffff))

/* end 3003.h */
