/* H8/300H-h8stub.c -- debugging stub for the Hitachi-H8/300H. 
*/

/*   This is originally based on an m68k software stub written by Glenn
     Engel at HP, but has changed quite a bit. 

     Modifications for the H8/300H by Steve Chamberlain
     Further modifications by Bill Knight - R O SoftWare
*/

/****************************************************************************

                THIS SOFTWARE IS NOT COPYRIGHTED

   HP offers the following for use in the public domain.  HP makes no
   warranty with regard to the software or it's performance and the
   user accepts the software "AS IS" with all faults.

   HP DISCLAIMS ANY WARRANTIES, EXPRESS OR IMPLIED, WITH REGARD
   TO THIS SOFTWARE INCLUDING BUT NOT LIMITED TO THE WARRANTIES
   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

****************************************************************************/


/* Remote communication protocol.

   A debug packet whose contents are <data>
   is encapsulated for transmission in the form:

        $ <data> # CSUM1 CSUM2

        <data> must be ASCII alphanumeric and cannot include characters
        '$' or '#'.  If <data> starts with two characters followed by
        ':', then the existing stubs interpret this as a sequence number.

        CSUM1 and CSUM2 are ascii hex representation of an 8-bit 
        checksum of <data>, the most significant nibble is sent first.
        the hex digits 0-9,a-f are used.

   Receiver responds with:

        +       - if CSUM is correct and ready for next packet
        -       - if CSUM is incorrect

   <data> is as follows:
   All values are encoded in ascii hex digits.

        Request         Packet

        read registers  g
        reply           XX....X         Each byte of register data
                                        is described by two hex digits.
                                        Registers are in the internal order
                                        for GDB, and the bytes in a register
                                        are in the same order the machine uses.
                        or ENN          for an error.

        write regs      GXX..XX         Each byte of register data
                                        is deSMRibed by two hex digits.
        reply           OK              for success
                        ENN             for an error

        write reg       Pn...=r...      Write register n... with value r...,
                                        which contains two hex digits for each
                                        byte in the register (target byte
                                        order).
        reply           OK              for success
                        ENN             for an error
        (not supported by all stubs).

        read mem        mAA..AA,LLLL    AA..AA is address, LLLL is length.
        reply           XX..XX          XX..XX is mem contents
                                        Can be fewer bytes than requested
                                        if able to read only part of the data.
                        or ENN          NN is errno

        write mem       MAA..AA,LLLL:XX..XX
                                        AA..AA is address,
                                        LLLL is number of bytes,
                                        XX..XX is data
        reply           OK              for success
                        ENN             for an error (this includes the case
                                        where only part of the data was
                                        written).

        cont            cAA..AA         AA..AA is address to resume
                                        If AA..AA is omitted,
                                        resume at same address.

        step            sAA..AA         AA..AA is address to resume
                                        If AA..AA is omitted,
                                        resume at same address.

        last signal     ?               Reply the current reason for stopping.
                                        This is the same reply as is generated
                                        for step or cont : SAA where AA is the
                                        signal number.

        There is no immediate reply to step or cont.
        The reply comes when the machine stops.
        It is           SAA             AA is the "signal number"

        or...           TAAn...:r...;n:r...;n...:r...;
                                        AA = signal number
                                        n... = register number
                                        r... = register contents
        or...           WAA             The process exited, and AA is
                                        the exit status.  This is only
                                        applicable for certains sorts of
                                        targets.
        kill request    k

        toggle debug    d               toggle debug flag (see 386 & 68k stubs)
        reset           r               reset -- see sparc stub.
        reserved        <other>         On other requests, the stub should
                                        ignore the request and send an empty
                                        response ($#<checksum>).  This way
                                        we can extend the protocol and GDB
                                        can tell whether the stub it is
                                        talking to uses the old or the new.
        search          tAA:PP,MM       Search backwards starting at address
                                        AA for a match with pattern PP and
                                        mask MM.  PP and MM are 4 bytes.
                                        Not supported by all stubs.

        general query   qXXXX           Request info about XXXX.
        general set     QXXXX=yyyy      Set value of XXXX to yyyy.
        query sect offs qOffsets        Get section offsets.  Reply is
                                        Text=xxx;Data=yyy;Bss=zzz
        console output  Otext           Send text to stdout.  Only comes from
                                        remote target.

        Responses can be run-length encoded to save space.  A '*' means that
        the next two characters are hex digits giving a repeat count which
        stands for that many repititions of the character preceding the '*'.
        Note that this means that responses cannot contain '*'.  Example:
        "0*03" means the same as "0000".  */


#include <string.h>
#include "h3003.h"
#include "h8stub.h"

#define BIT(n)  (1 << (n))
#define BREAKPOINT() asm(" trapa #3")

#define NMI_VEC           7
#define NMI1_VEC          9             /* Using trap 1 for user NMI */
#define USER_VEC          10            /* Using trap 2 for user I/O */
#define TRAP_VEC          11            /* Using trap 3 for breakpoints */


/*
 * Typedefs and enums
 */

enum regnames
{
  R0, R1, R2, R3, R4, R5, R6, R7,
  CCR, PC, NUM_REGS
};

#define NUMREGBYTES (4 * NUM_REGS)

/*
 * BUFMAX defines the maximum number of characters in inbound/outbound
 * buffers at least NUMREGBYTES * 2 are needed for register packets
 */
#define BUFMAX 400                      /* Same as value in gdb/remote.c */

/* how many times we'll try to send or receive */
#define MAX_GDB_PUTPACKET_TRIES 10

/*
 * Forward declarations
 */
static char highhex(int);
static char lowhex(int);
static int  hex(char);
static char *mem2hex(const char *, char *, int);
static char *hex2mem(char *, char *, int);
static int  hexToInt(char **, long *);
static void getpacket(char *);
static void putpacket_noack(char *);
static void putpacket(char *);
static void gdb_handle_exception(int);
static void handle_exception(int);
static int  oplen(unsigned char *);
static int  computeSignal(int);
static void InitGDBSerial(void);
static int  hw_rx_char(void);
static void hw_tx_char(int x);

/* number of WORDs of stack for use by stub */
#define stub_stack_size 512
static int  stub_stack[stub_stack_size] __attribute__ ((section (".stack"))) = {0};
static int  *stub_sp;

/* set aside an area in ram to
 * execute single instructions
 */
static unsigned char code[0x130];
static unsigned char enumber;

static short stepccr;
static long  old_sp;
static long  pcandccr;
static int   dofault;                   /* Non zero, bus errors will raise exception */

/* debug > 0 prints ill-formed commands in valid packets & checksum errors */
static int remote_debug;

static long  registers[NUM_REGS];
static const char hexchars[] = "0123456789abcdef";
static char  remcomInBuffer[BUFMAX];
static char  remcomOutBuffer[BUFMAX];


/*
 * Routines to handle hex data
 */

static char highhex(int x)
{
  return hexchars[(x >> 4) & 0xf];
}


static char lowhex(int x)
{
  return hexchars[x & 0xf];
}


static int hex(char ch)
{
  if ((ch >= 'a') && (ch <= 'f'))
    return (ch - 'a' + 10);

  if ((ch >= '0') && (ch <= '9'))
    return (ch - '0');

  if ((ch >= 'A') && (ch <= 'F'))
    return (ch - 'A' + 10);

  return (-1);
}


/* convert the memory, pointed to by mem into hex, placing result in buf */
/* return a pointer to the last char put in buf (null) */
static char *mem2hex(const char *mem, char *buf, int count)
{
  int i;
  int ch;

  for (i = 0; i < count; i++)
    {
    ch = *mem++;
    *buf++ = highhex(ch);
    *buf++ = lowhex(ch);
    }

  *buf = 0;
  return (buf);
}


/* convert the hex array pointed to by buf into binary, to be placed in mem */
/* return a pointer to the character after the last byte written */
static char *hex2mem(char *buf, char *mem, int count)
{
  int i;
  unsigned char ch;

  for (i = 0; i < count; i++)
    {
    ch = hex(*buf++) << 4;
    ch = ch + hex(*buf++);
    *mem++ = ch;
    }

  return (mem);
}


/**********************************************/
/* While we find nice HEX chars, build an INT */
/* Return number of chars processed           */
/**********************************************/
static int hexToInt(char **ptr, long *intValue)
{
  int numChars = 0;
  int hexValue;

  *intValue = 0;

  while (**ptr)
    {
    hexValue = hex(**ptr);

    if (hexValue >= 0)
      {
      *intValue = (*intValue << 4) | hexValue;
      numChars++;
      }
    else
      break;

    (*ptr)++;
    }

  return (numChars);
}


/*
 * Routines to get and put packets
 */

/* scan for the sequence $<data>#<checksum> */
static void getpacket(char *buffer)
{
  unsigned char checksum;
  unsigned char xmitcsum;
  int i;
  int count;
  char ch;

  do
    {
    /* wait around for the start character, ignore all other characters */
    while ((ch = hw_rx_char()) != '$')
      continue;

    checksum = 0;
    xmitcsum = -1;
    count = 0;

    /* now, read until a # or end of buffer is found */
    while (count < BUFMAX)
      {
      ch = hw_rx_char();

      if (ch == '#')
        break;

      checksum = checksum + ch;
      buffer[count++] = ch;
      }

      buffer[count] = 0;

      if (ch == '#')
        {
        xmitcsum = hex(hw_rx_char()) << 4;
        xmitcsum += hex(hw_rx_char());

        if (checksum != xmitcsum)
          hw_tx_char('-');              /* failed checksum */
        else
          {
          hw_tx_char('+');              /* successful transfer */

          /* if a sequence char is present, reply the sequence ID */
          if (buffer[2] == ':')
            {
            hw_tx_char(buffer[0]);
            hw_tx_char(buffer[1]);

            /* remove sequence chars from buffer */
            count = strlen(buffer);

            for (i = 3; i <= count; i++)
              buffer[i - 3] = buffer[i];
            }
          }
        }
    } while (checksum != xmitcsum);
}


/* Send the packet in buffer.  The host gets one chance to read it.
 * This routine does not wait for a positive acknowledge.
 */
static void putpacket_noack(register char *buffer)
{
  register short checksum;
  char *src = buffer;

  /*  $<packet info>#<checksum>. */
  hw_tx_char('$');
  checksum = 0;

  while (*src)
    {
    register short runlen;

    /* Do run length encoding */
    for (runlen = 0; runlen < 100; runlen++) 
      {
      if (src[0] != src[runlen]) 
        {
        if (runlen > 3) 
          {
          register int encode;

          /* Got a useful amount */
          hw_tx_char(*src);
          checksum += *src;
          hw_tx_char('*');
          checksum += '*';
          checksum += (encode = runlen + ' ' - 4);
          hw_tx_char(encode);
          src += runlen;
          }
        else
          {
          hw_tx_char(*src);
          checksum += *src;
          src++;
          }

        break;
        }
      }
    }

  hw_tx_char('#');
  hw_tx_char(highhex(checksum));
  hw_tx_char(lowhex(checksum));
}


/* Send the packet in buffer and wait for an acknowledgement.
 */
static void putpacket(char *buffer)
{
  short try_again = MAX_GDB_PUTPACKET_TRIES;

  while (try_again--)
    {
    putpacket_noack(buffer);

    if (hw_rx_char() == '+')
      break;
    }
}


/*
 * this function takes the H8/300H exception number and attempts to
 * translate this number into a unix compatible signal value
 */
static int computeSignal(int exceptionVector)
{
  int sigval;

  switch (exceptionVector)
    {
    case NMI1_VEC:
    case NMI_VEC:
      sigval = 2;
      break;    

    case TRAP_VEC:
    case USER_VEC:
    default:
      sigval = 5;
      break;
    }

  return (sigval);
}


/* return the length of the instuction
 * referenced at opaddr
 */
static int oplen(unsigned char *opaddr)
{
  static const char oplentable[] =
    {
    4, 2, 4, 2, 4, 2, 4, 2,             /* 0x58xx - 0x5Fxx */
    2, 2, 2, 2, 2, 2, 2, 2,             /* 0x60xx - 0x67xx */
    2, 2, 4, 4, 2, 2, 4, 4,             /* 0x68xx - 0x6Fxx */
    2, 2, 2, 2, 2, 2, 2, 2,             /* 0x70xx - 0x77xx */
    8, 4, 6, 4, 4, 4, 4, 4              /* 0x78xx - 0x7Fxx */
    };

  unsigned char op8;

  switch (op8 = opaddr[0])
    {
    case 0x01:
      switch (opaddr[1])
        {
        case 0xC0:                      /* MULXS */
        case 0xD0:                      /* DIVXS */
        case 0xF0:                      /* OR.L, XOR.L, AND.L */
          return 4;

        case 0x00:                      /* MOV.L */
        case 0x40:                      /* LDC(w), STC(w) */
          switch (opaddr[2])
            {
            case 0x69:
            case 0x6D:
              return 4;

            case 0x6F:
              return 6;

            case 0x6B:
              return (opaddr[3] & 0x20) ? 8 : 6;

            case 0x78:
              return 10;
            }

          break;
        }

      return 2;

    case 0x6A:                          /* MOV.B */
    case 0x6B:                          /* MOV.W */ 
      return (opaddr[1] & 0x20) ? 6 : 4;
    }

  if ((op8 < 0x58) ||
      (op8 > 0x7F))
    return 2;

  return (int)(oplentable[op8 - 0x58]);
}


/* "adjust" the program counter and
 * stack pointer to 24 bits, and the
 * condition code register to 8 bits
 */
static void prune(void)
{
  registers[R7]  &= 0xffffff;
  registers[PC]  &= 0xffffff;
  registers[CCR] &= 0x0000ff;
}


void astep(void);
void astepinhere(void);
void astepend(void);
  asm("
         .align 1
       astep:
       _astep:
         push.l  er0
         push.l  er1
         push.l  er2
         push.l  er3
         push.l  er4
         push.l  er5
         push.l  er6
         mov.l   er7,@_old_sp

      ;  make a copy of the ccr, because the ldc instruction
      ;  reads a word, but takes the byte on the even boundary

         mov.l   @_registers + (4 * 0),er0
         mov.l   @_registers + (4 * 1),er1
         mov.l   @_registers + (4 * 2),er2
         mov.l   @_registers + (4 * 3),er3
         mov.l   @_registers + (4 * 4),er4
         mov.l   @_registers + (4 * 5),er5
         mov.l   @_registers + (4 * 6),er6
         mov.l   @_registers + (4 * 7),er7
         ldc     @_stepccr, ccr
       astepinhere:
       _astepinhere:
         nop
         nop
         nop
         nop
         nop
         nop
         nop
         nop
         nop
         nop
         nop
  
         stc     ccr,@_stepccr
         mov.l   er0,@_registers + (4 * 0)
         mov.l   er1,@_registers + (4 * 1)
         mov.l   er2,@_registers + (4 * 2)
         mov.l   er3,@_registers + (4 * 3)
         mov.l   er4,@_registers + (4 * 4)
         mov.l   er5,@_registers + (4 * 5)
         mov.l   er6,@_registers + (4 * 6)
         mov.l   er7,@_registers + (4 * 7)

         mov.l   @_old_sp,er7
         pop.l   er6
         pop.l   er5
         pop.l   er4
         pop.l   er3
         pop.l   er2
         pop.l   er1
         pop.l   er0
         rts
       astepend:
       _astepend:
      ");

#define ASTEP_SIZE  ((unsigned long)astepend - (unsigned long)astep)
#define ASTEP_HERE  ((unsigned long)astepinhere - (unsigned long)astep)


/* declare prototype here so compiler
 * won't complain
 */
void hw_dostep(void);

void hw_dostep(void)
{
  unsigned  char *instrMem;
  long pc;
  long *sp;
  void (*call_code)(void);
  unsigned char *cp = code;
#define PUSH(x) { *--sp = x; }

  /* avoid some warnings */
  call_code = (void *)handle_exception;
  enumber = enumber;
  old_sp = old_sp;

  call_code = (void *)(code);

  prune();
  pc = registers[PC];
  sp = (long *)(registers[R7]);

  stepccr = registers[CCR] * 0x101;

  instrMem = (char *)registers[PC];

  if (((*instrMem & 0xf0) == 0x40) ||
      (*instrMem == 0x58))
    {
    long ifgone;
    long ifnotgone;
    long oldr0;

    /* BRA something, we hack the instruction to only jump
     * a little way and then run it, so BHI 1234 turns into
     * bhi .+4
     * adds #1,r0
     * and by inspecting r0 we can see if the branch happened
     */
    memcpy(cp, astep, ASTEP_SIZE);
    
    if (*instrMem == 0x58) 
      {
      ifgone = pc + ((instrMem[2] << 8) | (instrMem[3])) + 4;
      ifnotgone = pc + 4;
      cp[ASTEP_HERE + 0] = instrMem[0];
      cp[ASTEP_HERE + 1] = instrMem[1];
      cp[ASTEP_HERE + 2] = 0;
      cp[ASTEP_HERE + 3] = 8;
      cp[ASTEP_HERE + 4] = 0x0b;
      cp[ASTEP_HERE + 5] = 0x00;        /* ADDS #1,er0 */
      }
    else 
      {
      ifgone = pc + (signed char)(instrMem[1]) + 2;
      ifnotgone = pc + 2;

      /* A short branch */
      cp[ASTEP_HERE + 0] = instrMem[0];
      cp[ASTEP_HERE + 1] = 8;
      cp[ASTEP_HERE + 2] = 0x0b;
      cp[ASTEP_HERE + 3] = 0x00;        /* ADDS #1,er0 */
      }

    oldr0 = registers[R0];
    call_code();
    pc = (oldr0 == registers[R0]) ? ifgone : ifnotgone;
    registers[R0] = oldr0;
    }
  else 
    {
    int len = oplen((unsigned char *)pc);

    switch (*instrMem) 
      {
      int rd;
      long imm;

      case 0x54:                        /* RTS */
        if (instrMem[1] == 0x70)
          { 
          pc = *(long *)(registers[R7]);
          sp++;
          break;
          }

        goto def;

      case 0x55:                        /* BSR */
        PUSH(pc + len);
        pc += (signed char)(instrMem[1]) + 2;
        break;

      case 0x59:                        /* JMP @ERn */
        rd = (instrMem[1] >> 4) & 0x7;
        pc = registers[rd];
        break;

      case 0x5A:                        /* JMP abs */
        imm = ((long)instrMem[1] << 16) | (instrMem[2] << 8) | instrMem[3];
        pc = imm;
        break;

      case 0x5B:                        /* JMP @@val */
        imm = *(long *)((long)instrMem[1]);
        pc = imm;
        break;

      case 0x5C:                        /* BSR */
        if (instrMem[1] == 0) 
          {
          pc += ((instrMem[2] << 8) || instrMem[3]);
          break;
          }

        goto def;

      case 0x5D:                        /* JSR @ERn */
        rd = (instrMem[1] >> 4) & 0x7;
        PUSH(pc + len);
        pc = registers[rd];
        break;

      case 0x5E:                        /* JSR abs */
        imm = ((long)instrMem[1] << 16) | (instrMem[2] << 8) | instrMem[3];
        PUSH(pc + len);
        pc = imm;
        break;

      case 0x5F:                        /* JSR @@val */
        imm = *(long *)((long)instrMem[1]);
        PUSH(pc + len);
        pc = imm;
        break;

      default:
      def:
        /* Put the instruction inside a save and restore state and run it */
        len = oplen((unsigned char *)pc);
        memcpy(cp, astep, ASTEP_SIZE);
        memcpy(cp + ASTEP_HERE, instrMem, len);
        call_code();
        sp = (long *)(registers[R7]);
        pc += len;
        break;
      }
    }

  registers[PC] = pc;
  registers[R7] = (long)sp;
  registers[CCR] = stepccr;
  prune();
}


/*
 * This function does all exception handling.  It only does two things;
 * it figures out why it was called and tells gdb, and then it reacts
 * to gdb's requests.
*/
static void gdb_handle_exception(int exceptionVector)
{
  int sigval;
  long addr, length;
  char *ptr;

  /* Reply to host that an exception has occurred */
  sigval = computeSignal(exceptionVector);
  remcomOutBuffer[0] = 'S';
  remcomOutBuffer[1] = highhex(sigval);
  remcomOutBuffer[2] = lowhex(sigval);
  remcomOutBuffer[3] = 0;
  putpacket(remcomOutBuffer);

  /*
   * Test for a software trap inserted in place of
   * code.  If so, back up PC by one instruction,
   * since this instruction will later be replaced
   * by its original one!
   */
  if ((exceptionVector == TRAP_VEC) ||
      (exceptionVector == USER_VEC))
    {
    registers[PC] -= 2;
    }

  while (1)
    {
    remcomOutBuffer[0] = 0;
    getpacket(remcomInBuffer);

    switch (remcomInBuffer[0])
      {
      case '?':
        remcomOutBuffer[0] = 'S';
        remcomOutBuffer[1] = highhex(sigval);
        remcomOutBuffer[2] = lowhex(sigval);
        remcomOutBuffer[3] = 0;
        break;

      /* Toggle debug flag */
      case 'd':
        remote_debug = !(remote_debug);
        break;

      /* Return the value of the CPU registers */
      case 'g':
        mem2hex((char *)registers, remcomOutBuffer, NUMREGBYTES);
        break;

      /* Set the value of the CPU registers - return OK */
      case 'G':
        hex2mem(&remcomInBuffer[1], (char *)registers, NUMREGBYTES);
        strcpy(remcomOutBuffer, "OK");
        break;

      /* mAA..AA,LLLL  Read LLLL bytes at address AA..AA */
      case 'm':
        if (1)
          {
          dofault = 0;

          /* Try to read %x,%x.  If succeed, set ptr = 0 */
          ptr = &remcomInBuffer[1];

          if ((hexToInt(&ptr, &addr) != 0) &&
              (*(ptr++) == ',') &&
              (hexToInt(&ptr, &length) != 0))
            {
            ptr = 0;
            mem2hex((char *)addr, (char *)remcomOutBuffer, length);
            }

          if (ptr)
            strcpy(remcomOutBuffer, "E01");
          }
        else
          strcpy(remcomOutBuffer, "E03");

        /* Restore handler for bus error */
        dofault = 1;
        break;

      /* MAA..AA,LLLL: Write LLLL bytes at address AA.AA return OK */
      case 'M':
        if (1)        
          {
          dofault = 0;

          /* Try to read '%x,%x:'.  If succeed, set ptr = 0 */
          ptr = &remcomInBuffer[1];

          if ((hexToInt(&ptr, &addr) != 0) &&
              (*(ptr++) == ',') &&
              (hexToInt(&ptr, &length) != 0) &&
              (*(ptr++) == ':'))
            {
            hex2mem((char *)ptr, (char *) addr, length);
            ptr = 0;
            strcpy(remcomOutBuffer, "OK");
            }

          if (ptr)
            strcpy(remcomOutBuffer, "E02");
          }
        else
          strcpy(remcomOutBuffer, "E03");

        /* Restore handler for bus error */
        dofault = 1;
        break;

      /* cAA..AA   Continue at address AA..AA(optional) */
      /* sAA..AA   Step one instruction from AA..AA(optional) */
      case 'c':
      case 's':
        {
        /* Try to read optional parameter, PC unchanged if no parm */
        ptr = &remcomInBuffer[1];

        if (hexToInt(&ptr, &addr))
          registers[PC] = addr;

        if (remcomInBuffer[0] == 's') 
          {
          hw_dostep();
          remcomOutBuffer[0] = 'S';
          remcomOutBuffer[1] = '0';
          remcomOutBuffer[2] = '5';
          remcomOutBuffer[3] = 0;
          }
        else
          return;
        }

        break;

      /* Kill the program */
      case 'k':                         /* do nothing */
        break;
      }

    /* Reply to the request */
    putpacket(remcomOutBuffer);
    }
}


static void handle_exception(int exceptionVector)
{
  registers[PC] = pcandccr & 0xffffff;
  registers[CCR] = (pcandccr >> 24) & 0xff;
  registers[R7] &= 0xffffff;

  gdb_handle_exception(exceptionVector);

  pcandccr = (registers[PC] & 0xffffff) | (registers[CCR] << 24);
  registers[R7] &= 0xffffff;
}


/* void catch_exception(void); */

asm("
     _catch_exception:
       mov.b   r0l,@_enumber            /* save exception */

     ; check if gdbstub has been initialized

       mov.l   @_stub_sp,er0
       bne     _defaultvec
       rte

     _defaultvec:
       pop.l   er0                      /* get ccr + pc */
       mov.l   er0,@_pcandccr
       mov.l   er1,@_registers + 4 * 1
       mov.l   er2,@_registers + 4 * 2
       mov.l   er3,@_registers + 4 * 3
       mov.l   er4,@_registers + 4 * 4
       mov.l   er5,@_registers + 4 * 5
       mov.l   er6,@_registers + 4 * 6
       mov.l   er7,@_registers + 4 * 7

       mov.l   @_stub_sp,er7
       mov.b   @_enumber,r0l
       extu.w  r0
       jsr     @_handle_exception
       mov.l   @_registers + 4 * 7,er7  /* get user stack */
       mov.l   @_pcandccr,er0           /* fetch pc */
       push.l  er0                      /* stick onto stack */
       mov.l   @_registers + 4 * 6,er6
       mov.l   @_registers + 4 * 5,er5
       mov.l   @_registers + 4 * 4,er4
       mov.l   @_registers + 4 * 3,er3
       mov.l   @_registers + 4 * 2,er2
       mov.l   @_registers + 4 * 1,er1
       mov.l   @_registers + 4 * 0,er0
       rte

       .global _catch_e_1
     _catch_e_1:
       mov.l   er0,@_registers
       mov.b   #1,r0l
       .global _catch_e_2
     _catch_e_2:
       mov.l   er0,@_registers
       mov.b   #2,r0l
       bra     _catch_exception:16
       .global _catch_e_3
     _catch_e_3:
       mov.l   er0,@_registers
       mov.b   #3,r0l
       bra     _catch_exception:16
       .global _catch_e_4
     _catch_e_4:
       mov.l   er0,@_registers
       mov.b   #4,r0l
       bra     _catch_exception:16
       .global _catch_e_5
     _catch_e_5:
       mov.l   er0,@_registers
       mov.b   #5,r0l
       bra     _catch_exception:16
       .global _catch_e_6
     _catch_e_6:
       mov.l   er0,@_registers
       mov.b   #6,r0l
       bra     _catch_exception:16
       .global _catch_e_7
     _catch_e_7:
       mov.l   er0,@_registers
       mov.b   #7,r0l
       bra     _catch_exception:16
       .global _catch_e_8
     _catch_e_8:
       mov.l   er0,@_registers
       mov.b   #8,r0l
       bra     _catch_exception:16
       .global _catch_e_9
     _catch_e_9:
       mov.l   er0,@_registers
       mov.b   #9,r0l
       bra     _catch_exception:16
       .global _catch_e_10
     _catch_e_10:
       mov.l   er0,@_registers
       mov.b   #10,r0l
       bra     _catch_exception:16
       .global _catch_e_11
     _catch_e_11:
       mov.l   er0,@_registers
       mov.b   #11,r0l
       bra     _catch_exception:16
       .global _catch_e_12
     _catch_e_12:
       mov.l   er0,@_registers
       mov.b   #12,r0l
       bra     _catch_exception:16
       .global _catch_e_13
     _catch_e_13:
       mov.l   er0,@_registers
       mov.b   #13,r0l
       bra     _catch_exception:16
       .global _catch_e_14
     _catch_e_14:
       mov.l   er0,@_registers
       mov.b   #14,r0l
       bra     _catch_exception:16
       .global _catch_e_15
     _catch_e_15:
       mov.l   er0,@_registers
       mov.b   #15,r0l
       bra     _catch_exception:16
       .global _catch_e_16
     _catch_e_16:
       mov.l   er0,@_registers
       mov.b   #16,r0l
       bra     _catch_exception:16
       .global _catch_e_17
     _catch_e_17:
       mov.l   er0,@_registers
       mov.b   #17,r0l
       bra     _catch_exception:16
       .global _catch_e_18
     _catch_e_18:
       mov.l   er0,@_registers
       mov.b   #18,r0l
       bra     _catch_exception:16
       .global _catch_e_19
     _catch_e_19:
       mov.l   er0,@_registers
       mov.b   #19,r0l
       bra     _catch_exception:16
       .global _catch_e_20
     _catch_e_20:
       mov.l   er0,@_registers
       mov.b   #20,r0l
       bra     _catch_exception:16
       .global _catch_e_21
     _catch_e_21:
       mov.l   er0,@_registers
       mov.b   #21,r0l
       bra     _catch_exception:16
       .global _catch_e_22
     _catch_e_22:
       mov.l   er0,@_registers
       mov.b   #22,r0l
       bra     _catch_exception:16
       .global _catch_e_23
     _catch_e_23:
       mov.l   er0,@_registers
       mov.b   #23,r0l
       bra     _catch_exception:16
       .global _catch_e_24
     _catch_e_24:
       mov.l   er0,@_registers
       mov.b   #24,r0l
       bra     _catch_exception:16
       .global _catch_e_25
     _catch_e_25:
       mov.l   er0,@_registers
       mov.b   #25,r0l
       bra     _catch_exception:16
       .global _catch_e_26
     _catch_e_26:
       mov.l   er0,@_registers
       mov.b   #26,r0l
       bra     _catch_exception:16
       .global _catch_e_27
     _catch_e_27:
       mov.l   er0,@_registers
       mov.b   #27,r0l
       bra     _catch_exception:16
       .global _catch_e_28
     _catch_e_28:
       mov.l   er0,@_registers
       mov.b   #28,r0l
       bra     _catch_exception:16
       .global _catch_e_29
     _catch_e_29:
       mov.l   er0,@_registers
       mov.b   #29,r0l
       bra     _catch_exception:16
       .global _catch_e_30
     _catch_e_30:
       mov.l   er0,@_registers
       mov.b   #30,r0l
       bra     _catch_exception:16
       .global _catch_e_31
     _catch_e_31:
       mov.l   er0,@_registers
       mov.b   #31,r0l
       bra     _catch_exception:16
       .global _catch_e_32
     _catch_e_32:
       mov.l   er0,@_registers
       mov.b   #32,r0l
       bra     _catch_exception:16
       .global _catch_e_33
     _catch_e_33:
       mov.l   er0,@_registers
       mov.b   #33,r0l
       bra     _catch_exception:16
       .global _catch_e_34
     _catch_e_34:
       mov.l   er0,@_registers
       mov.b   #34,r0l
       bra     _catch_exception:16
       .global _catch_e_35
     _catch_e_35:
       mov.l   er0,@_registers
       mov.b   #35,r0l
       bra     _catch_exception:16
       .global _catch_e_36
     _catch_e_36:
       mov.l   er0,@_registers
       mov.b   #36,r0l
       bra     _catch_exception:16
       .global _catch_e_37
     _catch_e_37:
       mov.l   er0,@_registers
       mov.b   #37,r0l
       bra     _catch_exception:16
       .global _catch_e_38
     _catch_e_38:
       mov.l   er0,@_registers
       mov.b   #38,r0l
       bra     _catch_exception:16
       .global _catch_e_39
     _catch_e_39:
       mov.l   er0,@_registers
       mov.b   #39,r0l
       bra     _catch_exception:16
       .global _catch_e_40
     _catch_e_40:
       mov.l   er0,@_registers
       mov.b   #40,r0l
       bra     _catch_exception:16
       .global _catch_e_41
     _catch_e_41:
       mov.l   er0,@_registers
       mov.b   #41,r0l
       bra     _catch_exception:16
       .global _catch_e_42
     _catch_e_42:
       mov.l   er0,@_registers
       mov.b   #42,r0l
       bra     _catch_exception:16
       .global _catch_e_43
     _catch_e_43:
       mov.l   er0,@_registers
       mov.b   #43,r0l
       bra     _catch_exception:16
       .global _catch_e_44
     _catch_e_44:
       mov.l   er0,@_registers
       mov.b   #44,r0l
       bra     _catch_exception:16
       .global _catch_e_45
     _catch_e_45:
       mov.l   er0,@_registers
       mov.b   #45,r0l
       bra     _catch_exception:16
       .global _catch_e_46
     _catch_e_46:
       mov.l   er0,@_registers
       mov.b   #46,r0l
       bra     _catch_exception:16
       .global _catch_e_47
     _catch_e_47:
       mov.l   er0,@_registers
       mov.b   #47,r0l
       bra     _catch_exception:16
       .global _catch_e_48
     _catch_e_48:
       mov.l   er0,@_registers
       mov.b   #48,r0l
       bra     _catch_exception:16
       .global _catch_e_49
     _catch_e_49:
       mov.l   er0,@_registers
       mov.b   #49,r0l
       bra     _catch_exception:16
       .global _catch_e_50
     _catch_e_50:
       mov.l   er0,@_registers
       mov.b   #50,r0l
       bra     _catch_exception:16
       .global _catch_e_51
     _catch_e_51:
       mov.l   er0,@_registers
       mov.b   #51,r0l
       bra     _catch_exception:16
       .global _catch_e_52
     _catch_e_52:
       mov.l   er0,@_registers
       mov.b   #52,r0l
       bra     _catch_exception:16
       .global _catch_e_53
     _catch_e_53:
       mov.l   er0,@_registers
       mov.b   #53,r0l
       bra     _catch_exception:16
       .global _catch_e_54
     _catch_e_54:
       mov.l   er0,@_registers
       mov.b   #54,r0l
       bra     _catch_exception:16
       .global _catch_e_55
     _catch_e_55:
       mov.l   er0,@_registers
       mov.b   #55,r0l
       bra     _catch_exception:16
       .global _catch_e_56
     _catch_e_56:
       mov.l   er0,@_registers
       mov.b   #56,r0l
       bra     _catch_exception:16
       .global _catch_e_57
     _catch_e_57:
       mov.l   er0,@_registers
       mov.b   #57,r0l
       bra     _catch_exception:16
       .global _catch_e_58
     _catch_e_58:
       mov.l   er0,@_registers
       mov.b   #58,r0l
       bra     _catch_exception:16
       .global _catch_e_59
     _catch_e_59:
       mov.l   er0,@_registers
       mov.b   #59,r0l
       bra     _catch_exception:16
      .global _catch_e_60
     _catch_e_60:
       mov.l   er0,@_registers
       mov.b   #60,r0l
       bra     _catch_exception:16
       .global _catch_e_61
     _catch_e_61:
       mov.l   er0,@_registers
       mov.b   #61,r0l
       bra     _catch_exception:16
      .global _catch_e_62
     _catch_e_62:
       mov.l   er0,@_registers
       mov.b   #62,r0l
       bra     _catch_exception:16
      .global _catch_e_63
     _catch_e_63:
       mov.l   er0,@_registers
       mov.b   #63,r0l
       bra     _catch_exception:16
    ");


void InitGDBStub(void)
{
  InitGDBSerial();

  dofault = 1;

  stub_sp = stub_stack + stub_stack_size;

  BREAKPOINT();
}


void breakpoint(void)
{
  if (stub_sp)
    BREAKPOINT();
}
  

/* output buffer to gdb console.
 * returns -1 if buffer too long
 * 0 otherwise
 */
short gdb_console_output(const char *buf, int len)
{
  /* make sure it will all fit */
  if (len > ((BUFMAX * 2) - 2))
    return -1;

  remcomOutBuffer[0] = 'O';
  mem2hex(buf, &remcomOutBuffer[1], len);
  putpacket_noack(remcomOutBuffer);
  return 0;
}


/* output string to gdb console.
 * returns -1 if buffer too long
 * 0 otherwise
 */
short gdb_console_puts(const char *buf)
{
  return gdb_console_output(buf, strlen(buf));
}


#if (GDBPORT == SCI_0) || (GDBPORT == SCI_1)
/* Using H8-300H internal serial ports */

/*
 * Serial mode register bits
 */
#define SYNC_MODE       BIT(7)
#define SEVEN_BIT_DATA  BIT(6)
#define PARITY_ON       BIT(5)
#define ODD_PARITY      BIT(4)
#define STOP_BITS_2     BIT(3)
#define ENABLE_MULTIP   BIT(2)
#define PHI_64          (BIT(1) | BIT(0))
#define PHI_16          BIT(1)
#define PHI_4           BIT(0)

/*
 * Serial control register bits
 */
#define SCI_TIE         BIT(7)          /* Transmit interrupt enable */
#define SCI_RIE         BIT(6)          /* Receive interrupt enable */
#define SCI_TE          BIT(5)          /* Transmit enable */
#define SCI_RE          BIT(4)          /* Receive enable */
#define SCI_MPIE        BIT(3)          /* Multiprocessor interrupt enable */
#define SCI_TEIE        BIT(2)          /* Transmit end interrupt enable */
#define SCI_CKE1        BIT(1)          /* Clock enable 1 */
#define SCI_CKE0        BIT(0)          /* Clock enable 0 */

/*
 * Serial status register bits
 */
#define SCI_TDRE        BIT(7)          /* Transmit data register empty */
#define SCI_RDRF        BIT(6)          /* Receive data register full */
#define SCI_ORER        BIT(5)          /* Overrun error */
#define SCI_FER         BIT(4)          /* Framing error */
#define SCI_PER         BIT(3)          /* Parity error */
#define SCI_TEND        BIT(2)          /* Transmit end */
#define SCI_MPB         BIT(1)          /* Multiprocessor bit */
#define SCI_MPBT        BIT(0)          /* Multiprocessor bit transfer */


#define SMR_PORT        (GDBPORT ? SMR1 : SMR0)
#define SCR_PORT        (GDBPORT ? SCR1 : SCR0)
#define BRR_PORT        (GDBPORT ? BRR1 : BRR0)
#define RDR_PORT        (GDBPORT ? RDR1 : RDR0)
#define SSR_PORT        (GDBPORT ? SSR1 : SSR0)
#define TDR_PORT        (GDBPORT ? TDR1 : TDR0)


static void nop(void)
{
  return;
}


static void InitGDBSerial(void)
{
  int i;

  /* Clear TE and RE in the SCR */
  SCR_PORT = 0x00;

  /* Set communication to be async, 8-bit data,
   * no parity, 1 stop bit and use internal clock
   */
  SMR_PORT = 0x00;
  BRR_PORT = (CPU_HZ / (32.0 * GDBBAUD)) - .5; // let compiler do fractional rounding

  /* Wait for one bit */
  for (i = 0; i < 10000; i++)
    nop();

  SCR_PORT |= (SCI_RE | SCI_TE);
}


static int __inline__ hw_rx_ready(void)
{
  /* check for & clear any comm errors */
  if (SSR_PORT & (SCI_PER | SCI_FER | SCI_ORER))
    SSR_PORT &= ~(SCI_ORER | SCI_PER | SCI_FER);

  return (SSR_PORT & SCI_RDRF);
}


static int hw_rx_char(void)
{
  unsigned int ch;

  /* wait for receive data available */
  while (!hw_rx_ready())
    continue;

  ch = (unsigned int)RDR_PORT;
  SSR_PORT &= ~SCI_RDRF;

  /* check for & clear any comm errors */
  if (SSR_PORT & (SCI_PER | SCI_FER | SCI_ORER))
    SSR_PORT &= ~(SCI_ORER | SCI_PER | SCI_FER);

  return (int)ch;
}


static void hw_tx_char(int ch)
{
  /* wait until last transmit data complete */
  while (!(SSR_PORT & SCI_TDRE))
    continue;

  /*
   * Write data into TDR and clear TDRE
   */
  TDR_PORT = (unsigned char)ch;
  SSR_PORT &= ~SCI_TDRE;
}

#elif (GDBPORT == AICOM)
/* Using PromICE's AI virtual serial port */

static void InitGDBSerial(void)
{
  unsigned char data;

  /* wait for PromICE to initialize */
  while (*AI_STATUS == 0xCC)
    continue;

  /* dummy read */
  data = *AI_DATA;
}


static int hw_rx_char(void)
{
  /* wait for host data available */
  while (!(*AI_STATUS & HDA))
    continue;

  /* return data received */
  return (unsigned int)*AI_DATA;
}


static void hw_tx_char(int ch)
{
  int count;
  unsigned char data;

  /* wait until last transmit data complete */
  while (*AI_STATUS & TDA)
    continue;

  /* send start bit */
  data = *AI_ONE;

  /* loop through the 8 bits of data */
  for (count = 8; count; ch >>= 1, count--)
    data = (ch & 0x01) ? *AI_ONE : *AI_ZERO;

  /* send stop bit */
  data = *AI_ONE;
}

#elif (GDBPORT == VCOM)
/* Using Tech-Tool's VCOM virtual serial port */

static void InitGDBSerial(void)
{
  /* nothing to do */
}


static int hw_rx_char(void)
{
  unsigned int ch;

  /* wait for receive data available */
  while (*VCOM_RXFLAG == 0)
    continue;

  ch = *VCOM_RXDATA;                    /* read data */
  *VCOM_RXFLAG = 0;                     /* indicate data has been read */
  return (int)ch;
}


static void hw_tx_char(int ch)
{
  /* wait for last transmit data complete */
  while (*VCOM_TXFLAG)
    continue;

  *VCOM_TXBUFF = (unsigned char)ch;     /* output data */
  *VCOM_TXFLAG = 1;                     /* indicate new data in output buffer */
}

#endif
