#ifndef INC_H8STUB_C
#define INC_H8STUB_C


/* initialize the gdb stub and BREAK
 */
void InitGDBStub(void);


/* used to insert a BREAKPOINT into
 * your code
 */
void breakpoint(void);


/* output buffer to gdb console.
 * returns -1 if buffer too long
 * 0 otherwise
 */
short gdb_console_output(const char *buf, int len);


/* output string to gdb console.
 * returns -1 if buffer too long
 * 0 otherwise
 */
short gdb_console_puts(const char *buf);


/* Define the various serial port types
 * NOTE: SCI_0 must not be moved from 0
 */
#define SCI_0 0                         /* CPU's SCI port 0 */
#define SCI_1 1                         /* CPU's SCI port 1 */
#define AICOM 2                         /* PromICE's AI */
#define VCOM  3                         /* Tech-Tool's VCOM */


/**************************************************************
 * GDBPORT - USER CONFIGURATION PARAMETERS
 **************************************************************/
#define GDBPORT     (AICOM)


#if (GDBPORT == SCI_0) || (GDBPORT == SCI_1) /* H8-300H SCI Port */

/**************************************************************
 * MHZ & GDBBAUD - USER CONFIGURATION PARAMETERS
 **************************************************************/
#define CPU_HZ      (16e6)              /* speed of CPU in Hz (float is OK) */
#define GDBBAUD     (38400)             /* communications data rate */

#elif (GDBPORT == AICOM)                /* PromICE's AI */

/**************************************************************
 * BUS_SIZE & AICOM_BASE - USER CONFIGURATION PARAMETERS
 **************************************************************/
#define BUS_SIZE    8
#define AICOM_BASE  0x00FFF8L

#define TDA         0x01                /* Target Data Available */
#define HDA         0x02                /* Host Data Available */
#define OVR         0x04                /* Host Data Overflow */

#if (BUS_SIZE == 8)
#define AI_ZERO     (volatile unsigned char *)(AICOM_BASE + 0)
#define AI_ONE      (volatile unsigned char *)(AICOM_BASE + 1)
#define AI_DATA     (volatile unsigned char *)(AICOM_BASE + 2)
#define AI_STATUS   (volatile unsigned char *)(AICOM_BASE + 3)
#elif (BUS_SIZE == 16)
#define AI_ZERO     (volatile unsigned char *)(AICOM_BASE + 0)
#define AI_ONE      (volatile unsigned char *)(AICOM_BASE + 2)
#define AI_DATA     (volatile unsigned char *)(AICOM_BASE + 4)
#define AI_STATUS   (volatile unsigned char *)(AICOM_BASE + 6)
#endif

#elif (GDBPORT == VCOM)                 /* Tech-Tool's VCOM */

/**************************************************************
 * VCOM_BASE - USER CONFIGURATION PARAMETER
 **************************************************************/
#define VCOM_BASE 0x00FFF8L

#define VCOM_TXBUFF (volatile unsigned char *)(VCOM_BASE + 0)
#define VCOM_TXFLAG (volatile unsigned char *)(VCOM_BASE + 1)
#define VCOM_RXFLAG (volatile unsigned char *)(VCOM_BASE + 2)
#define VCOM_RXDATA (volatile unsigned char *)(VCOM_BASE + 3)

#endif


#endif
